package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.config.CryptConfiguer;
import com.diagbot.dto.ConceptWithOrderRes;
import com.diagbot.dto.PartDTO;
import com.diagbot.dto.PartSymptomDTO;
import com.diagbot.entity.Concept;
import com.diagbot.entity.KlSymptom;
import com.diagbot.entity.Symptom;
import com.diagbot.entity.wrapper.ConceptWrapper;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconRSTypeEnum;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.CryptUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.PartSymptomVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 部位facade
 * @Author: ztg
 * @Date: 2018/10/23 16:33
 */
@Component
public class PartFacade {

    @Autowired
    ConceptFacade conceptFacade;
    @Autowired
    ConceptDetailFacade conceptDetailFacade;
    @Autowired
    private CryptConfiguer cryptConfiguer;
    @Autowired
    KlSymptomFacade klSymptomFacade;

    /**
     * 根据已选部位返回对应的症状
     *
     * @param partSymptomVO 参数
     * @return 部位症状关联信息
     */
    public List<PartSymptomDTO> getSymptomByPartName(PartSymptomVO partSymptomVO) {
        List<String> partList = partSymptomVO.getPartList();
        if (ListUtil.isEmpty(partList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                    "请选择部位");
        }
        if (cryptConfiguer.cryptFlag) {
            CryptUtil.encryptList(partList);
        }
        if (partSymptomVO.getSexType() == null) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                    "请选择性别");
        }
        if (partSymptomVO.getAge() == null) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                    "请选择年龄");
        }
        if (partSymptomVO.getAge().intValue() < 0 || partSymptomVO.getAge().intValue() > 200) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR,
                    "超出年龄范围【0 — 200】");
        }
        List<PartSymptomDTO> res = new ArrayList<>();

        // 获取部位区域
        List<Concept> conceptList = conceptFacade.list(new QueryWrapper<Concept>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("lib_type", LexiconTypeEnum.PART_AREA.getKey())
                .in("lib_name", partList)
        );

        // 正常数据有2条，将全身区域放到最下面
        String areaStr = cryptConfiguer.cryptFlag ?
                CryptUtil.encrypt_char( "全身区域") : "全身区域";
        if (ListUtil.isNotEmpty(conceptList) && conceptList.size() == 2
                && areaStr.equals(conceptList.get(0).getLibName())) {
            Collections.reverse(conceptList);
        }

        // 设置部位区域
        for (Concept c : conceptList) {
            PartSymptomDTO partSymptomDTO = new PartSymptomDTO();
            partSymptomDTO.setConceptId(c.getId());
            partSymptomDTO.setName(c.getLibName());
            res.add(partSymptomDTO);
        }

        // 获取部位信息
        ConceptWrapper conceptWrapper = new ConceptWrapper();
        conceptWrapper.setStartNameList(partList);
        conceptWrapper.setStartType(LexiconTypeEnum.PART_AREA.getKey());
        conceptWrapper.setRelationType(LexiconRSTypeEnum.ORDER_BY.getKey());
        conceptWrapper.setEndType(LexiconTypeEnum.BODYPART.getKey());
        conceptWrapper.setEndSex(partSymptomVO.getSexType());
        conceptWrapper.setEndAge(partSymptomVO.getAge());
        List<ConceptWithOrderRes> list = conceptFacade.getConceptWithOrder(conceptWrapper);

        Map<Long, List<ConceptWithOrderRes>> partMap = EntityUtil.makeEntityListMap(list, "startId");
        // 设置部位区域下的部位
        for (PartSymptomDTO partSymptomDTO : res) {
            List<ConceptWithOrderRes> part = partMap.get(partSymptomDTO.getConceptId());
            if (ListUtil.isNotEmpty(part)) {
                List<PartDTO> partDTOS = new ArrayList<>();
                for (ConceptWithOrderRes conceptWithOrderRes : part) {
                    PartDTO partDTO = new PartDTO();
                    partDTO.setName(conceptWithOrderRes.getEndName());
                    partDTO.setConceptId(conceptWithOrderRes.getEndId());
                    partDTOS.add(partDTO);
                }
                partSymptomDTO.setPartDTO(partDTOS);
            }
        }

        // 获取部位下的症状
        ConceptWrapper wrapper = new ConceptWrapper();
        wrapper.setStartNameList(list.stream().map(r -> r.getEndName()).distinct().collect(Collectors.toList()));
        wrapper.setStartType(LexiconTypeEnum.BODYPART.getKey());
        wrapper.setRelationType(LexiconRSTypeEnum.ORDER_BY.getKey());
        wrapper.setEndType(LexiconTypeEnum.SYMPTOM.getKey());
        wrapper.setEndSex(partSymptomVO.getSexType());
        wrapper.setEndAge(partSymptomVO.getAge());
        List<ConceptWithOrderRes> symptomList = conceptFacade.getConceptWithOrder(wrapper);
        Map<Long, List<ConceptWithOrderRes>> symptomMap = EntityUtil.makeEntityListMap(symptomList, "startId");


        // TODO 获取症状的简述信息
        List<KlSymptom> klSymptomList = klSymptomFacade.list(new QueryWrapper<KlSymptom>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("concept_id",
                        symptomList.stream().map(row -> row.getEndId()).distinct().collect(Collectors.toList()))
        );
        Map<Long, KlSymptom> descMap = klSymptomList.stream().collect(Collectors.toMap(row -> row.getConceptId(), row -> row));

        // 设置部位下的症状
        for (PartSymptomDTO partSymptomDTO : res) {
            List<PartDTO> partDTOS = partSymptomDTO.getPartDTO();
            for (PartDTO partDTO : partDTOS) {
                List<ConceptWithOrderRes> symptom = symptomMap.get(partDTO.getConceptId());
                if (ListUtil.isNotEmpty(symptom)) {
                    List<Symptom> symptoms = new ArrayList<>();
                    for (ConceptWithOrderRes conceptWithOrderRes : symptom) {
                        Symptom bean = new Symptom();
                        bean.setConceptId(conceptWithOrderRes.getEndId());
                        bean.setName(conceptWithOrderRes.getEndName());
                        bean.setPartConceptId(conceptWithOrderRes.getStartId());
                        if (descMap.get(conceptWithOrderRes.getEndId()) != null) {
                            bean.setDescription(descMap.get(conceptWithOrderRes.getEndId()).getDescription());
                            bean.setExplains(descMap.get(conceptWithOrderRes.getEndId()).getExplains());
                        }
                        symptoms.add(bean);
                    }
                    partDTO.setSymptomList(symptoms);
                }
            }
        }

        // 如部位无关联的症状，则删除部位
        for (PartSymptomDTO dto : res) {
            for (int i = 0; i < dto.getPartDTO().size(); i++) {
                PartDTO partDTO = dto.getPartDTO().get(i);
                if (ListUtil.isEmpty(partDTO.getSymptomList())) {
                    dto.getPartDTO().remove(i--);
                }
            }
        }

        return res;
    }

}
