package com.diagbot.web;


import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.ConceptRetrievalDTO;
import com.diagbot.dto.ConceptRetrievalExtDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.UsualDTO;
import com.diagbot.entity.Concept;
import com.diagbot.facade.ConceptFacade;
import com.diagbot.util.ParamConvertUtil;
import com.diagbot.vo.ConceptBaseVO;
import com.diagbot.vo.ConceptExistVO;
import com.diagbot.vo.ConceptFindVO;
import com.diagbot.vo.ConceptSearchVO;
import com.diagbot.vo.ConceptUsualVO;
import com.diagbot.vo.GetStaticKnowledgeVO;
import com.diagbot.vo.RetrievalVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 概念表 前端控制器
 * </p>
 *
 * @author zhaops
 * @since 2019-05-06
 */
@RestController
@RequestMapping("/concept")
@Api(value = "常用标签API", tags = { "知识库标准化-常用标签API" })
@SuppressWarnings("unchecked")
public class ConceptController {
    @Autowired
    ConceptFacade conceptFacade;

    @ApiOperation(value = "知识库标准化-常用标签——根据科室获取常用标签[by:wangyu]",
            notes = "deptName：科室名称<br>" +
                    "age：年龄<br>" +
                    "sexType: 性别<br>" +
                    "type：类型(1：症状 3：其他史 4：查体，5：化验 6：辅检 7：诊断)，必填<br>")
    @PostMapping("/getConceptUsual")
    @SysLogger("getConceptUsual")
    public RespDTO<List<ConceptBaseDTO>> getConceptUsual(@RequestBody @Valid ConceptUsualVO conceptUsualVO) {
        List<ConceptBaseDTO> data = conceptFacade.getConceptUsual(conceptUsualVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "知识库标准化-常用标签——根据科室获取常用标签包含通俗解释和详细说明[by:zhoutg]",
            notes = "deptName：科室名称<br>" +
                    "age：年龄<br>" +
                    "sexType: 性别<br>" +
                    "type：类型(1：症状 3：其他史 4：查体，5：化验 6：辅检 7：诊断)，必填<br>")
    @PostMapping("/getConceptExtUsual")
    @SysLogger("getConceptExtUsual")
    public RespDTO<List<UsualDTO>> getConceptExtUsual(@RequestBody @Valid ConceptUsualVO conceptUsualVO) {
        List<UsualDTO> data = conceptFacade.getConceptExtUsual(conceptUsualVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 根据概念Id列表获取概念列表Map
     *
     * @param conceptSearchVO 搜索参数
     * @return 术语id和术语 Map
     */
    @ApiOperation(value = "知识库标准化-根据概念Id列表获取概念列表Map[by:gaodm]",
            notes = "conceptIds: 术语Id列表")
    @PostMapping("/getConceptMap")
    @SysLogger("getConceptMap")
    public RespDTO<Map<String, Long>> getConceptMap(@RequestBody ConceptSearchVO conceptSearchVO) {
        return RespDTO.onSuc(conceptFacade.getConceptMap(conceptSearchVO));
    }

    /**
     * 根据概念Id列表获取概念列表Map
     *
     * @param conceptFindVO 搜索参数
     * @return 术语id和术语 Map
     */
    @ApiOperation(value = "知识库标准化-根据概念Id,年龄,性别列表获取概念列表Map[by:gaodm]",
            notes = "conceptIds: 术语Id列表" +
                    "age：年龄<br>" +
                    "sexType: 性别<br>")
    @PostMapping("/getConceptCondMap")
    @SysLogger("getConceptCondMap")
    public RespDTO<Map<Long, String>> getConceptCondMap(@RequestBody ConceptFindVO conceptFindVO) {
        return RespDTO.onSuc(conceptFacade.getConceptCondMap(conceptFindVO));
    }

    /**
     * 验证存在并且获取概念基本信息
     *
     * @param conceptBaseVO
     * @return
     */
    @ApiOperation(value = "知识库标准化-验证存在并且获取概念基本信息[by:zhaops]",
            notes = "name: 概念名称<br>" +
                    "libType: 词性<br>")
    @PostMapping("/exist")
    @SysLogger("exist")
    @ApiIgnore
    public RespDTO<ConceptBaseDTO> exist(@RequestBody ConceptBaseVO conceptBaseVO) {
        conceptBaseVO.setLibType(ParamConvertUtil.conceptConvert2Lib(conceptBaseVO.getLibType()));
        Concept concept = conceptFacade.exist(conceptBaseVO);
        ConceptBaseDTO conceptBaseDTO = new ConceptBaseDTO();
        conceptBaseDTO.setName(concept.getLibName());
        conceptBaseDTO.setConceptId(concept.getId());
        return RespDTO.onSuc(conceptBaseDTO);
    }

    @ApiOperation(value = "知识库标准化-标签检索[by:wangyu]",
            notes = "type：类型(1：症状 3：其他史 4：查体，5：化验 6：辅检 7：诊断)，必填<br>" +
                    "age：年龄，必填<br>" +
                    "inputStr：检索内容，必填<br>" +
                    "sexType：性别，必填<br>" +
                    "inputIds：需要去重的id<br>")
    @PostMapping("/retrivelConceptInfo")
    @SysLogger("retrivelConceptInfo")
    public RespDTO<List<ConceptRetrievalDTO>> retrivelConceptInfo(@RequestBody @Valid RetrievalVO retrievalVO) {
        List<ConceptRetrievalDTO> data = conceptFacade.retrivelConceptInfo(retrievalVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "知识库标准化-标签检索,附带通俗解释和详细说明[by:zhoutg]",
            notes = "type：类型(1：症状 3：其他史 4：查体，5：化验 6：辅检 7：诊断)，必填<br>" +
                    "age：年龄，必填<br>" +
                    "inputStr：检索内容，必填<br>" +
                    "sexType：性别，必填<br>" +
                    "inputIds：需要去重的id<br>")
    @PostMapping("/retrivelConceptInfoExt")
    @SysLogger("retrivelConceptInfoExt")
    public RespDTO<List<ConceptRetrievalExtDTO>> retrivelConceptInfoExt(@RequestBody @Valid RetrievalVO retrievalVO) {
        List<ConceptRetrievalExtDTO> data = conceptFacade.retrivelConceptInfoExt(retrievalVO);
        return RespDTO.onSuc(data);
    }

    /**
     * 根据名称和类型获取概念列表Map
     *
     * @param conceptExistVO 搜索参数
     * @return 术语id和术语 Map
     */
    @ApiOperation(value = "知识库标准化-根据名称和类型获取概念列表Map[by:zhoutg]",
            notes = "nameList: 名称列表<br>" +
                    "type: 类型")
    @PostMapping("/getConceptMapByNameAndType")
    @SysLogger("getConceptMapByNameAndType")
    public RespDTO<Map<String, Long>> getConceptMapByNameAndType(@RequestBody ConceptExistVO conceptExistVO) {
        return RespDTO.onSuc(conceptFacade.getConceptMap(conceptExistVO));
    }

    @ApiOperation(value = "ICSS三期-静态知识-页面检索[by:wangyu]",
            notes = "inputStr：检索内容，必填<br>" +
                    "inputIds：需要去重的id<br>")
    @PostMapping("/getStaticKnowledge")
    @SysLogger("getStaticKnowledge")
    public RespDTO<List<RetrievalDTO>> getStaticKnowledge(@Valid @RequestBody GetStaticKnowledgeVO getStaticKnowledgeVO) {
        List<RetrievalDTO> data = conceptFacade.getStaticKnowledge(getStaticKnowledgeVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "静态知识检索【海南】[by:zhoutg]",
            notes = "inputStr：检索内容，必填<br>" +
                    "inputIds：需要去重的id<br>" +
                    "types：指定类型，1：症状，10：药品，12：化验，16：辅检，18：诊断，25：手术，33：体征，" +
                    "72：诊疗指南（门诊）,73：诊疗指南（住院）,74：护理,75：临床路径,76：血液,77：麻醉,78：监护," +
                    "79：中医,80：入科流程,81：审查规则,82：电子书,83：视频， 不指定：以上全部")
    @PostMapping("/getStaticKnowledgeHaiNan")
    @SysLogger("getStaticKnowledgeHaiNan")
    public RespDTO<List<RetrievalDTO>> getStaticKnowledgeHaiNan(@Valid @RequestBody GetStaticKnowledgeVO getStaticKnowledgeVO) {
        List<RetrievalDTO> data = conceptFacade.getStaticKnowledgeHaiNan(getStaticKnowledgeVO);
        return RespDTO.onSuc(data);
    }
}
