package com.diagbot.util;

/**
 * @Description: 实体类加解密工具类
 * @author: gaodm
 * @date: 2019/12/31 21:07
 * @version: V1.0
 */

import com.diagbot.annotation.CryptField;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Field;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class CryptPojoUtils {

    /**
     * 对含注解字段加密
     */
    public static <T> void encryptFields(T[] objects) {
        for (Object obj : objects) {
            encryptFieldOrList(obj);
        }
    }

    /**
     * 对含注解字段加密
     */
    public static <T> void encryptFieldOrList(T t) {
        if (isNotCrypt(t)) {
            return;
        }
        if (t instanceof List) {
            List values = (List) t;
            for (Object object : values) {
                CryptPojoUtils.encryptField(object);
            }
        } else {
            CryptPojoUtils.encryptField(t);
        }
    }

    /**
     * 对含注解字段加密
     */
    private static <T> void encryptField(T t) {
        if (isNotCrypt(t)) {
            return;
        }

        Field[] declaredFields = getAllFields(t.getClass());
        try {
            if (declaredFields != null && declaredFields.length > 0) {
                for (Field field : declaredFields) {
                    if (field.isAnnotationPresent(CryptField.class)
                            && field.getType().toString().endsWith("String")) {
                        field.setAccessible(true);
                        String fieldValue = (String) field.get(t);
                        if (StringUtils.isNotEmpty(fieldValue)) {
                            field.set(t, CryptUtil.encrypt_char(fieldValue));
                        }
                    } else if (field.getType().equals(List.class)) {
                        field.setAccessible(true);
                        List fieldValue = (List) field.get(t);
                        if (ListUtil.isEmpty(fieldValue)) {
                            continue;
                        }
                        for (Object listValue : fieldValue) {
                            encryptField(listValue);
                        }
                    } else if (field.getType().equals(LinkedList.class)) {
                        field.setAccessible(true);
                        LinkedList fieldValue = (LinkedList) field.get(t);
                        if (ListUtil.isEmpty(fieldValue)) {
                            continue;
                        }
                        for (Object listValue : fieldValue) {
                            encryptField(listValue);
                        }
                    } else {
                        field.setAccessible(true);
                        Object fieldValue = field.get(t);
                        encryptField(fieldValue);
                    }
                }
            }
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对含注解字段解密
     */
    public static <T> void decryptFields(T[] objects) {
        for (Object obj : objects) {
            decryptFieldOrList(obj);
        }
    }

    /**
     * 对含注解字段解密
     */
    public static <T> void decryptFieldOrList(T t) {
        if (isNotCrypt(t)) {
            return;
        }
        if (t instanceof List) {
            List values = (List) t;
            for (Object object : values) {
                CryptPojoUtils.decryptField(object);
            }
        } else {
            CryptPojoUtils.decryptField(t);
        }
    }

    /**
     * 对含注解字段解密
     */
    private static <T> void decryptField(T t) {
        if (isNotCrypt(t)) {
            return;
        }

        Field[] declaredFields = getAllFields(t.getClass());
        try {
            if (declaredFields != null && declaredFields.length > 0) {
                for (Field field : declaredFields) {
                    if (field.isAnnotationPresent(CryptField.class)
                            && field.getType().toString().endsWith("String")) {
                        field.setAccessible(true);
                        String fieldValue = (String) field.get(t);
                        if (StringUtils.isNotEmpty(fieldValue)) {
                            field.set(t, CryptUtil.decrypt_char(fieldValue));
                        }
                    } else if (field.getType().equals(List.class)) {
                        field.setAccessible(true);
                        List fieldValue = (List) field.get(t);
                        if (ListUtil.isEmpty(fieldValue)) {
                            continue;
                        }
                        for (Object listValue : fieldValue) {
                            decryptField(listValue);
                        }
                    } else if (field.getType().equals(LinkedList.class)) {
                        field.setAccessible(true);
                        LinkedList fieldValue = (LinkedList) field.get(t);
                        if (ListUtil.isEmpty(fieldValue)) {
                            continue;
                        }
                        for (Object listValue : fieldValue) {
                            decryptField(listValue);
                        }
                    } else {
                        field.setAccessible(true);
                        Object fieldValue = field.get(t);
                        decryptField(fieldValue);
                    }
                }
            }
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }


    /**
     * 判断是否需要加解密
     *
     * @param o
     * @return
     */
    private static boolean isNotCrypt(Object o) {
        return o == null || o instanceof Double
                || o instanceof Integer || o instanceof Long
                || o instanceof Boolean || o instanceof Map
                || o instanceof Date || o instanceof LocalDate
                || o instanceof MultipartFile || o instanceof HttpServletRequest
                || o instanceof HttpServletResponse;
    }

    /**
     * 获取本类及其父类的属性的方法
     *
     * @param clazz 当前类对象
     * @return 字段数组
     */
    private static Field[] getAllFields(Class<?> clazz) {
        List<Field> fieldList = new ArrayList<>();
        while (clazz != null) {
            fieldList.addAll(new ArrayList<>(Arrays.asList(clazz.getDeclaredFields())));
            clazz = clazz.getSuperclass();
        }
        Field[] fields = new Field[fieldList.size()];
        return fieldList.toArray(fields);
    }
}
