package com.diagbot.web;

import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.GraDTO;
import com.diagbot.dto.GraphLabelDTO;
import com.diagbot.dto.NodeDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SchemaDTO;
import com.diagbot.dto.TreeDTO;
import com.diagbot.facade.KgFacade;
import com.diagbot.vo.KgQueryVO;
import com.diagbot.vo.KgTreeVO;
import com.diagbot.vo.SchemaVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * @Description: 朗通知识图谱控制层
 * @author: gaodm
 * @time: 2018/8/30 10:12
 */
@RestController
@RequestMapping("/kg")
@Api(value = "朗通知识图谱API", tags = { "朗通知识图谱API" })
@SuppressWarnings("unchecked")
@Slf4j
public class KgController {
    private static final String KGTREECACHE = "KgTreeCache";

    @Autowired
    private KgFacade kgFacade;

    @ApiOperation(value = "查询节点", notes = "查询节点")
    @PostMapping("/getNode")
    @SysLogger("getNode")
    public RespDTO<List<NodeDTO>> getNode(@RequestBody @Valid KgQueryVO kgQueryVO) {
        return RespDTO.onSuc(kgFacade.getNode(kgQueryVO));
    }

    @ApiOperation(value = "获取图谱", notes = "获取图谱")
    @PostMapping("/getGraph")
    @SysLogger("getGraph")
    public RespDTO<GraphLabelDTO> getGraph(@RequestBody @Valid KgQueryVO kgQueryVO) {
        return RespDTO.onSuc(kgFacade.getGraphFac(kgQueryVO));
    }

    @ApiOperation(value = "获取树形分类",
            notes = "获取树形分类(疾病ICD10：t：1，st：1)<br>" +
                    "(疾病科室：t：1，st：2)<br>" +
                    "(药品：t：2，st：0)<br>" +
                    "(症状：t：3，st：0)<br>" +
                    "(手术和操作：t：4，st：0)<br>" +
                    "(实验室检查：t：5，st：0)<br>" +
                    "(辅助检查：t：6，st：0)<br>")
    @PostMapping("/getTree")
    @SysLogger("getTree")
    @Cacheable(value = KGTREECACHE, key = "'kgtree:t_' + #kgTreeVO.type + '_st_' + #kgTreeVO.subType")
    public RespDTO<TreeDTO> getTree(@RequestBody @Valid KgTreeVO kgTreeVO) {
        log.info("获取树形分类缓存成功");
        return RespDTO.onSuc(kgFacade.getTreeFac(kgTreeVO));
    }

    @ApiOperation(value = "获取描述框架", notes = "获取描述框架")
    @PostMapping("/getSchema")
    @SysLogger("getSchema")
    public RespDTO<List<SchemaDTO>> getSchema(@RequestBody @Valid SchemaVO schemaVO) {
        return RespDTO.onSuc(kgFacade.getSchema(schemaVO));
    }

    @ApiOperation(value = "获取图", notes = "获取图")
    @PostMapping("/getTu")
    @SysLogger("getTu")
    @ApiIgnore
    public RespDTO<GraDTO> getTu(@RequestBody KgQueryVO kgQueryVO) {
        return RespDTO.onSuc(kgFacade.getTuFac(kgQueryVO));
    }


    @ApiOperation(value = "清理树形分类缓存", notes = "清理树形分类缓存")
    @PostMapping("/delKgTreeCache")
    @SysLogger("delKgTreeCache")
    @CacheEvict(cacheNames = KGTREECACHE, allEntries = true)
    public RespDTO<Boolean> delKgTreeCache() {
        log.info("清理树形分类缓存成功");
        return RespDTO.onSuc(true);
    }
}

